#version 330 core

//Interpolierte Werte vom Vertexshader uebergeben
in vec2 fUVs;
in vec3 position_wspc;
in vec3 normal_cspc;
in vec3 viewdir_cspc;
in vec3 lightdir_cspc;

//Output (Farbe des Pixels)
out vec4 outcolor;

//Konstante Werte frs ganze Mesh
uniform sampler2D colorTexture;
uniform vec3 lPosition_wspc;

void main(){

	//Lichtwerte (hardgecoded, spaeter ueber in bestimmbar)
	vec3 lColor= vec3(1,1,1);
	float lPower= 50.0f;

	//Materialwerte 
	vec3 mDiffuseColor= texture(colorTexture, fUVs).rgb;
	vec3 mAmbientColor= vec3(0.1, 0.1, 0.1) * mDiffuseColor;
	vec3 mSpecularColor= vec3(0.3, 0.3, 0.3);

	//Distanz zum Licht
	float distance= length(lPosition_wspc - position_wspc);

	//Normale des zu berechnenden Fragments(N), im Cameraspace
	vec3 n= normalize(normal_cspc);

	//Richtung des Lichtes(L)
	vec3 l= normalize(lightdir_cspc);

	//Cosinus des Winkels zwischen der Normale und der Lichtrichtung(cosNL)
	//Falls der Winkel negativ ist wird er auf 0,1 gesetzt 
	float cosNL= clamp( dot(n,l), 0,1);

	//Viewvektor 
	vec3 V= normalize(viewdir_cspc);
	//Halbvektor H
	vec3 H= reflect(-l,n); //normalize(V + l);
	
	float cosNH= clamp( dot(n,H), 0,1);
	
	vec3 color=
	//Ambient: simuliert indirekte Beleuchtung
	mAmbientColor +
	//Diffuse: "Farbe" des Objektes
	mDiffuseColor * lColor * lPower * cosNL/(distance*distance) +
	//Specular: reflektierter Glanzpunkt
	mSpecularColor * lColor * lPower * pow(cosNH,5)/(distance*distance);
	outcolor=vec4(color, 1.0);
}