#version 330 core

//Input der Vertices
in vec3 position;
in vec2 uv;
in vec3 normal;

//Output, wird an den Fragment uebergeben, dh. interpolation pro Fragment
out vec2 fUVs;
out vec3 position_wspc;
out vec3 normal_cspc;
out vec3 viewdir_cspc;
out vec3 lightdir_cspc;

//Konstante Werte fuer das ganze mesh.
uniform mat4 viewMatrix;
uniform mat4 modelMatrix;
uniform mat4 projectionMatrix;
uniform vec3 lPosition_wspc;

void main(){

	//Outputposition des Vertex (outPos), im Clipspace: modelviewprojection Matrix * position
	mat4 MVP= projectionMatrix * viewMatrix * modelMatrix;
	gl_Position= MVP * vec4(position,1);

	//Position des Vertex (P), im Worldspace: modelmatrix * position
	position_wspc= (modelMatrix * vec4(position,1)).xyz;

	//Vektor der vom Vertex auf die Kamera zeigt (viewMatrix), im Cameraspace
	// im Cameraspace ist die Kamera im Ausgangspunkt
	vec3 vPosition_cspc= (viewMatrix * modelMatrix * vec4(position,1)).xyz;
	viewdir_cspc= vec3(0,0,0) - vPosition_cspc;

	//Vektor der vom Vertex auf die Lichtquelle zeigt (_light)
	vec3 lPosition_cspc= (viewMatrix * vec4(lPosition_wspc,1)).xyz;
	lightdir_cspc= lPosition_cspc + viewdir_cspc;

	//Normale des Vertex (N), in Cameraspace
	normal_cspc= (viewMatrix * modelMatrix * vec4(normal,0)).xyz; //Nur richtig wenn die Modelmatrix das model nicht skaliert, sonst die inverse transponierte Matrix.

	//UV des Vertex 
	fUVs= uv;
}