#version 330 core

//Interpolierte Werte vom Vertexshader uebergeben
in vec2 fUVs;
in vec4 position_wspc;
in vec3 normal_cspc;
in vec3 viewdir_cspc;
in vec4 tangent_cspc;

//Output (Farbe des Pixels)
out vec4 outcolor;

// Licht


struct lighting{
	vec3 position;
	vec3 color;
	float power;
	vec2 near_far;
	float unit;
};

layout (std140) uniform Material {
	vec4 diffuse;
	vec4 ambient;
	vec4 specular;
	vec4 emissive;
	float shininess;
	int texCount;
};

// Texturen
uniform sampler2D colorTexture;
uniform sampler2D normal_map;
uniform samplerCube depth_map;
uniform samplerCube depth_map2;
uniform samplerCube depth_map3;
uniform samplerCube depth_map4;
uniform samplerCube depth_map5;


uniform mat4 modelMatrix;
uniform mat4 viewMatrix;
uniform lighting light[5];
uniform int numberoflights; 

//if 1 true, if 0 false
uniform int normalMapping;
uniform int shadowMapping;

vec3 gridSamplingDisk[20] = vec3[]
(
   vec3(1, 1, 1), vec3(1, -1, 1), vec3(-1, -1, 1), vec3(-1, 1, 1), 
   vec3(1, 1, -1), vec3(1, -1, -1), vec3(-1, -1, -1), vec3(-1, 1, -1),
   vec3(1, 1, 0), vec3(1, -1, 0), vec3(-1, -1, 0), vec3(-1, 1, 0),
   vec3(1, 0, 1), vec3(-1, 0, 1), vec3(1, 0, -1), vec3(-1, 0, -1),
   vec3(0, 1, 1), vec3(0, -1, 1), vec3(0, -1, -1), vec3(0, 1, -1)
);

float evaluate_shadow(int i) {
	float result=1.0;
	float epsilon=0.15;	
	int numOfSamples=4;

	for(int a=0;a<numOfSamples;a++){
		int index = a;
		vec3 cm_lookup_vec= position_wspc.xyz - light[i].position;
		float curr_fragment_dist_to_light= length(cm_lookup_vec);
		float diskRadius = (1.0 + (1.0 - curr_fragment_dist_to_light) * 3) / 512; // /SizeofCubeTex
		
		float smallest_dist_to_light=1.0;
		vec3 baseDirection= cm_lookup_vec;
		vec3 baseOffset= gridSamplingDisk[a] * diskRadius;

		if(light[i].unit==2){
			smallest_dist_to_light= texture(depth_map, baseDirection+baseOffset).r;
		}
		if(light[i].unit==3){
			smallest_dist_to_light= texture(depth_map3,baseDirection+baseOffset).r;
		}
		if(light[i].unit==4){
			smallest_dist_to_light= texture(depth_map2,baseDirection+baseOffset).r;
		}
		if(light[i].unit==5){
			smallest_dist_to_light= texture(depth_map4,baseDirection+baseOffset).r;
		}
		if(light[i].unit==6){
			smallest_dist_to_light= texture(depth_map5,baseDirection+baseOffset).r;
		}
		smallest_dist_to_light *=light[i].near_far.y;

		if(smallest_dist_to_light+epsilon< curr_fragment_dist_to_light){
		result-=0.9/numOfSamples;
		}
	}

	return result;

}

void main(){

	vec3 mDiffuseColor;
	vec3 mAmbientColor;
	vec3 mSpecularColor;

	// Normal Map Berechnungen

	// normal map
	vec3 NewNormal;
	NewNormal = normal_cspc;

	if(normalMapping == 1){
	vec3 Normal = normal_cspc;
	vec3 Tangent = tangent_cspc.xyz;
	Tangent = normalize(Tangent - dot(Tangent, Normal) * Normal);
	vec3 Bitangent = cross(Normal,Tangent);
	Bitangent = normalize(Bitangent * tangent_cspc.w);
	vec3 normal_tan = texture (normal_map, fUVs).rgb;
	normal_tan = normalize (normal_tan * 2.0 - 1.0);
	 
	 mat3 TBN = mat3(Tangent, Bitangent, Normal);
	 NewNormal = TBN*normal_tan;
	 NewNormal = normalize(NewNormal);
	}
	// normal map samplen und von 0:1 auf -1:1 konvertieren

	// Shadow Map Berechnungen

	float shadow_factor = 1.0;

	

	// Materialwerte
	 
	//if(texCount == 0){
	//	mDiffuseColor= diffuse.rgb;
	//	mAmbientColor= ambient.rgb;
	//	mSpecularColor= specular.rgb;
	//} else {
		mDiffuseColor= texture(colorTexture, fUVs).rgb;
		mAmbientColor= vec3(0.1, 0.1, 0.1) * mDiffuseColor;
		mSpecularColor= vec3(0.3, 0.3, 0.3);
	//}

	// Phong Berechnungen

	vec3 color = vec3(0.0, 0.0, 0.0);
	
	for(int i=0; i<numberoflights; i++){
		//Lichtwerte (hardgecoded, spaeter ueber in bestimmbar)
	
		//Lichtwerte werden ausgelesen
		vec3 lColor= light[i].color;
		float lPower= light[i].power;

		vec3 lPosition_cspc= vec3(viewMatrix * vec4(light[i].position,1.0));
		vec3 lightdir_cspc= lPosition_cspc + viewdir_cspc;

		

		//Distanz zum Licht
		float distance= length(light[i].position - position_wspc.xyz);

		//Normale des zu berechnenden Fragments(N), im Cameraspace
		vec3 n = normalize(NewNormal);

		//Richtung des Lichtes(L)
		vec3 l= normalize(lightdir_cspc);

		//Cosinus des Winkels zwischen der Normale und der Lichtrichtung(cosNL)
		//Falls der Winkel negativ ist wird er auf 0,1 gesetzt 
		float cosNL= clamp( dot(l,n), 0.0,1.0);


		vec3 E= normalize(viewdir_cspc);
		vec3 R= reflect(-l,n);
		float cosAlpha= clamp(dot(E,R), 0.0,1.0);

		if(shadowMapping == 1){
		shadow_factor = evaluate_shadow(i);
		color+=		
		//Diffuse: "Farbe" des Objektes
		 shadow_factor *(mDiffuseColor * lColor * lPower * cosNL/(distance*distance)) +
		//Specular: reflektierter Glanzpunkt
		 shadow_factor *(mSpecularColor * lColor * lPower * pow(cosAlpha,5.0)/(distance*distance));
		} else {
		color+=		
		//Diffuse: "Farbe" des Objektes
		mDiffuseColor * lColor * lPower * cosNL/(distance*distance) +
		//Specular: reflektierter Glanzpunkt
		mSpecularColor * lColor * lPower * pow(cosAlpha,5.0)/(distance*distance);
		}
	}

	//Ambient: simuliert indirekte Beleuchtung
	color+= mAmbientColor;
	outcolor=vec4(color, 1.0);
}


